$(function () {
    /**
     *  Optional Confirmation Modal

     ## PURPOSE
     On next-button-click this script checks a list of dataLabels and if those (optional) questions
     were answered and if not, it displays a modal, asking for confirmation on skipping those questions.

     The click on the next-button is intercepted using a copy of the next button.
     This makes this script incompatible with other the-next-button-manipulating scripts.


     ## USAGE
     This needs
      - a html element that loads this script
      - a html element below that that contains the modal content and will be auto-hidden

     Drop the following script block into the first HTML element.

     ```
     <script src="modal.v7.2.confirmoptional.1.se.js"></script>
     <script id="confirmOptionalScript">
        $(function(){
            SePlay.Confirmoptional({
                dataLabels: ["s3","q4"] // data labels for all the elements to check if answered before NEXT
            }); // more see defaultOptions for parameters
        })
     </script>

     ```


     The 2nd HTML element on that page will be hidden, on next-button-click  and on-next-click displayed in a popup.

     @version 7.2.confirmoptional.2
     @date 2019-10-24
     @author abernh <a.bernhard@23go.de>

     ## CHANGELOG
        2019-10-27 v7.2.confirmoptional.2
            - added matrix support

        2019-10-24 v7.2.confirmoptional.1
            - added next-button-clone

        2019-10-24 v7.1.confirmoptional.1
            - created

     */

    // SETTINGS

    var
        VERSION = "7.2.confirmoptional.2",
        LOG_LABEL = "SE_NXTMDL",
        NS = "confirmOptionalModal",

        defaultOptions = {
            thisId: "confirmOptionalScript",

            debug: false,

            buttonStyles: {
                "font-size": "1.2em"
            },

            onCancel: function(onNextConditionResult){

                $('body,html').scrollTop($(onNextConditionResult).offset().top);

            },
            onNextCondition: function(){ //
                var emptyNodes = window.querySelectorAll('input[value=""].optional');
                if(emptyNodes.length){
                    return emptyNodes[0];
                }

                return;
            },

            labelConfirm: "SKIP",
            labelCancel: "ANSWER"
        };


    // SETTINGS -- end -------------------------------------------------------------------------

    var
        Nextmodal = function(){
            var _this = this;

            _this.v = {
                destroyQueue: [],
                isInit: false
            };

            return function(options){
                if(options === "destroy"){
                    _this.destroy.apply(_this, [options])
                }else{
                    _this.init.apply(_this, [options])
                }
            }
        },
        _proto = Nextmodal.prototype;

    _proto.version = VERSION;

    _proto.init = function(options){

        options = options || {};
        for(var i in defaultOptions){
            if(typeof(options[i]) === "undefined"){
                options[i] = defaultOptions[i];
            }
        }

        this.hideNextItem(options.thisId);

        if(this.v.isInit){
            return;
        }
        this.v.isInit = true;

        this.options = options;
        this.createModalButton();
    };

    _proto.hideNextItem = function(thisId){
        var _this = this,
            $thisItem = $('#' + thisId).closest('.se_item'),
            $nextItem = $thisItem.next();

        // HIDING NEXT ITEM
        if(!$('#modal' + NS + 'ContentStyles').length)
            $('body').append('<style id="modal' + NS + 'ContentStyles">.hidden' + NS + 'Content{ display: none !important; }</style>');

        $nextItem.css('display', 'none!important').addClass('hidden' + NS + 'Content');
    };

    _proto.createModalButton = function(){
        var _this = this,
            options = this.options,
            $thisItem = $('#' + options.thisId).closest('.se_item'),
            $nextItem = $thisItem.next(),

            onNextConditionResult,
            hasConfirmed;

        // GET ELEMENTS
        var
            $footer = $('.se_footer'),
            $page = $('.se_item').parent(),
            $form = $('form[name=play]'),

            $modal = $('<div id="se_' + NS + '">'+
                '<div style="position:fixed;top:0;left:0;right:0;bottom:0;background:rgba(0,0,0,0.5);z-index:999;">' +
                '<div style="display:flex;align-items:center;justify-content:center;height:100%;width:100%;">' +
                '<div style="width:400px;max-width:90%;padding:1em;border:1px solid gray;background:white;">' +
                '<div class="modal_body"></div>' +
                '<div class="modal_footer" style="margin-top:1em;border-top:1px solid gray;padding-top:1em;">' +
                '<button class="btn btn-default modal_button_close">' + options.labelCancel + '</button>' +
                '<button class="btn btn-primary modal_button_confirm" style="float:right;">' + options.labelConfirm + '</button>' +
                '</div></div></div></div></div>'
            ),
            $modalBody = $('.modal_body', $modal),
            $modalBtnCancel = $('.modal_button_close', $modal),
            $modalBtnConfirm = $('.modal_button_confirm', $modal),

            $nextButton = $('#se_next_button'),
            $nextButtonClone = $nextButton.clone();

        $nextButton.hide();
        $nextButtonClone.insertAfter($nextButton);



        // SETUP MODAL
        $modal.hide();
        $modalBody.html($nextItem.find('.se_text > div').html());

        $('body #se_' + NS + '').remove(); // remove previous left-over modals.
        $('body').append($modal);
        this.v.destroyQueue.push(function(){
            $modal.remove();
        });

        // SETUP BUTTONS
        $modalBtnCancel.css(options.buttonStyles);
        $modalBtnCancel.on('click', function () {
            $modal.hide();
            options.onCancel(onNextConditionResult);
        });
        $modalBtnConfirm.css(options.buttonStyles);
        $modalBtnConfirm.on('click', function () {
            $(this).attr('disabled', true);

            hasConfirmed = true;

            // trigger -> next
            $nextButton.trigger('click');

            _this.destroy();
        });

        $nextButtonClone.on('click', function(ev){
            if(hasConfirmed) return true;

            onNextConditionResult = options.onNextCondition();

            if(onNextConditionResult){
                $modal.show();

            }else{
                hasConfirmed = true;
                $nextButton.trigger('click');
                _this.destroy();
            }
        })


    };

    _proto.destroy = function(){
        var destroyCb = this.v.destroyQueue.pop();
        while(typeof(destroyCb) === "function"){
            destroyCb.apply(this);
            destroyCb = this.v.destroyQueue.pop();
        }
        this.v.isInit = false;
    };


    var SePlay = window.SePlay = window.SePlay || {};

    if(!SePlay.Confirmoptional){
        var nextModal = new Nextmodal(),
            Confirmoptional = function(params){

                if(!params.dataLabels ||
                    !params.dataLabels.length){
                    console.error(LOG_LABEL, "missing parameter 'dataLabels' array");
                    return;
                }

                var CONST_MULTIPLE = "multiple",
                    CONST_DISCRETE = "discrete",
                    CONST_SCALAR = "scalar",
                    CONST_M_MULTIPLE = "matrix multiple",
                    CONST_M_DISCRETE = "matrix discrete",
                    CONST_M_SCALAR = "matrix scalar";

                var
                    onNextConditionFn = function(){
                        for(var i in params.dataLabels){
                            if(!params.dataLabels.hasOwnProperty(i)) continue;

                            var dataLabel = params.dataLabels[i],
                                $seItem = $('#se_item_' + dataLabel),
                                dataType =
                                    $seItem.is(".se_itemtype_multiple") ? CONST_MULTIPLE :   // multiple choice
                                    $seItem.is(".se_itemtype_discrete") ? CONST_DISCRETE :   // single choice
                                    $seItem.is(".se_itemtype_scalar") ?  CONST_SCALAR :       // free text
                                    $seItem.is(".se_itemtype_association") ? CONST_M_MULTIPLE :   // matrix multiple choice
                                    $seItem.is(".se_itemtype_matrix") ? CONST_M_DISCRETE :   // matrix single choice
                                    $seItem.is(".se_itemtype_openmatrix") ?  CONST_M_SCALAR :       // matrix free text
                                    'other',
                                isAnswered,
                                $fields;

                            switch(dataType){
                                case CONST_MULTIPLE: // checkboxes
                                case CONST_M_MULTIPLE: // checkboxes
                                    // find a single checkbox that was checked -> is answered
                                    $fields = $seItem.find('input[type="checkbox"]').filter(function(i, elm){
                                        return elm.checked;
                                    });

                                    isAnswered = $fields.length > 0;
                                    break;
                                case CONST_DISCRETE: // radio
                                case CONST_M_DISCRETE: // radio
                                    // find a single radio that is checked -> is answered
                                    $fields = $seItem.find('input[type="radio"]').filter(function(i, elm){
                                        return elm.checked;
                                    });

                                    isAnswered = $fields.length > 0;
                                    break;
                                case CONST_SCALAR: // textarea / input[text]
                                case CONST_M_SCALAR: // textarea / input[text]
                                    // find a single text field with value -> is answered
                                    $fields = $seItem.find("textarea, input[type='text']").filter(function(i, elm){
                                        return $(elm).val().trim() !== "";
                                    });

                                    isAnswered = $fields.length > 0;
                                    break;
                                default:
                                    console.error(LOG_LABEL, 'unsupported data type for given dataLabel', dataLabel);
                                    break;
                            }

                            if(!isAnswered){
                                return $seItem;
                            }
                        }
                    };

                return nextModal({
                    onNextCondition: onNextConditionFn
                })

            };


        SePlay.Confirmoptional = Confirmoptional;
    }

});