!function($){

    /**
     * SePlay.Modal
     *
     * simple modal script that creates a bootstrap based modal with an individual content
     *
     * example for "hide next element and create modal from it"
     *
     * ```
        this is my <button id="openModal0">button0</button>

        <script src="se.modal.v1.js"></script>
        <script id="seModal0">
            $(function(){
                var $this = $('#seModal0').closest('.se_item'),
                    $next = $this.next(),
                    modal = new SePlay.Modal({
                        title: "My title",
                        body: $next.find('.se_text')
                    });

                $('#openModal0').on('click', function(){ modal.show(); });
            })
        </script>
       ```
     *
     *
     *
     */

    var modalTemplate = '' +
        '<div class="modal fade inmodal">' +
            '<div class="modal-dialog">' +
                '<div class="modal-content">' +
                    '<div class="modal-header">' +
                        '<button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">×</span></button> ' +
                        '<h4 class="modal-title">Settings</h4>' +
                    '</div>' +
                    '<div class="modal-body form-horizontal" style="overflow: auto; max-height: 70vh;"> ' +
                    '</div>' +
                    '<div class="modal-footer">' +
                        '<button type="button" class="btn btn-default" data-dismiss="modal">Cancel</button>' +
                        '<button type="button" class="btn btn-primary" data-positive="1" data-dismiss="modal"><i class="fa fa-floppy-o"></i>Ok</button>' +
                    '</div>' +
                '</div>' +
            '</div>' +
        '</div>';


    var SeModal = function(params){
            var _this = this;

            var v = {
                params: {
                    title: "title",
                    body: "body - either a selector or an html string",
                    labelConfirm: "labelConfirm",
                    onConfirm: function(){},
                    noHeader: false
                },
                elms: {
                    $modal: null,
                    $modalContent: $(params.body)
                }
            };

            this.createModal = function(){
                var $modal = $(modalTemplate);

                $modal.hide();
                if(!v.params.title){
                    v.params.title = "";
                }
                $modal.find('.modal-header h4').text(v.params.title);
                $modal.find('.modal-body').html(v.elms.$modalContent.length ? v.elms.$modalContent.html() : params.body);

                $modal.modal({
                    backdrop: false,
                    show: false
                });

                if(v.params.confirmOnly){
                    $modal.find('.modal-footer button.btn-default').hide();
                }

                if(v.params.noHeader){
                    $modal.find('.modal-header').hide();
                }

                $btnConfirm = $modal.find('.modal-footer button[data-positive=1]');
                $btnConfirm
                    .text(v.params.labelConfirm)
                    .on('click', v.params.onConfirm);

                v.elms.$modal = $modal;
                $('body').append($modal);

            };

            this.destroyModal = function(){};


            this.init = function(){
                _this.createModal();
            };

            this.show = function(){
                if(!v.elms.$modal) {
                    _this.init();
                }
                v.elms.$modal.modal('show');

            };
            this.hide = function(){
                v.elms.$modal.modal('hide');
            };



            for(var i in params){
                v.params[i] = params[i];
            }

            if(typeof params.body === "object")
                $(params.body).hide();

            return {
                init: this.init,
                show: this.show,
                hide: this.hide
            };
        };

    var SePlay = window.SePlay = window.SePlay || {};

    SePlay.Modal = SeModal;

}(jQuery);